local LibOrangUtils = LibStub:NewLibrary("LibOrangUtils-1.0", 5)

if not LibOrangUtils then
	do return end -- already loaded and no upgrade necessary
end

-- replace string(s) (with whole-word option)
function LibOrangUtils:str_replace(source, find, replace, wholeword)
  if wholeword then
    find = '%f[%a]'..find..'%f[%A]'
  end
  return (source:gsub(find,replace))
end

-- escape special/magic characters (regex) in a string
function LibOrangUtils:str_escape(str)
	return (str:gsub('[%-%.%+%[%]%(%)%$%^%%%?%*]','%%%1'):gsub('%z','%%z'))
end

-- split string by pattern/delimiter
-- eg: mytable = str_split("test1:test2:test3", "\:")
function LibOrangUtils:str_split(str, del)
	-- http://stackoverflow.com/a/1579673

	local ret = {}
	local fpat = "(.-)" .. del
	local last_end = 1
	local s, e, cap = str:find(fpat, 1)
	
	while s do
		if s ~= 1 or cap ~= "" then
			table.insert(ret,cap)
		end
	
		last_end = e+1
		s, e, cap = str:find(fpat, last_end)
	end
	
	if last_end <= #str then
		cap = str:sub(last_end)
		table.insert(ret, cap)
	end
	
	return ret
end

-- split text at length into table
function LibOrangUtils:str_lensplit(text, atLength)
	local ret = {}
	
	local maxChars 		= atLength or 1000
	local text_len 		= string.len(text);
	
	if(text_len <= maxChars) then
		ret[#ret+1] = text
	else
		for from = 0, text_len, maxChars do
			local to = from+maxChars-1
			
			if(from >= text_len) then from = text_len end
			if(to >= text_len) then to = text_len end
		
			--ret = ret+1
			ret[#ret+1] = string.sub(text, from, to)
		end
	end
	
	return ret
end	

-- cleanup string (duplicated whitespaces n' stuff)
function LibOrangUtils:str_cleanup(str)

	-- Remove leading and trailing whitespace(s)
	str = str:gsub("^%s*(.-)%s*$", "%1")

	-- Remove any duplicated whitespace(s)
	str = str:gsub("%s%s+", " ")
	
	return str
end

-- round number to the given number of decimal places
function LibOrangUtils:round(num, decimal)
	local exp = decimal and 10^decimal or 1
	return math.ceil(num * exp - 0.5) / exp
end

function LibOrangUtils:CopyToClipboard(text, autoSplit)
	if(not text or type(text) ~= "string") then do return false end end	
	
	local autoSplit = autoSplit or true
	
	if(not _G["LOUTils_CLIPBOARD"]) then

		WINDOW_MANAGER:CreateTopLevelWindow("LOUTils_CLIPBOARD")
			
		if(not _G["LOUTils_CLIPBOARD_CONTENT"]) then
			WINDOW_MANAGER:CreateControlFromVirtual("LOUTils_CLIPBOARD_CONTENT", _G["LOUTils_CLIPBOARD"], "ZO_DefaultEditMultiLineForBackdrop")
			_G["LOUTils_CLIPBOARD_CONTENT"]:SetAlpha(0)
			_G["LOUTils_CLIPBOARD_CONTENT"]:SetCopyEnabled(true)
			_G["LOUTils_CLIPBOARD_CONTENT"]:SetMaxInputChars(1000)
			_G["LOUTils_CLIPBOARD_CONTENT"]:SetMouseEnabled(false)
		end
	end	
	
	-- partial copy & paste
	if(string.len(text) > 1000 and autoSplit) then
	
		if(not _G["LOUTils_CLIPBOARD_ALERT"]) then
			local ALERT = WINDOW_MANAGER:CreateTopLevelWindow("LOUTils_CLIPBOARD_ALERT")
			ALERT:SetDimensions(400, 140)
			ALERT:SetAnchor(CENTER, GuiRoot, CENTER)
			ALERT:SetClampedToScreen(true)
			ALERT:SetMouseEnabled(true)
			ALERT:SetMovable(true)
			ALERT:SetTopmost(true)
			ALERT:SetHidden(false)
			
			-- BG
			WINDOW_MANAGER:CreateControlFromVirtual("LOUTils_CLIPBOARD_ALERT_BG", ALERT, "ZO_DefaultBackdrop")
			
			-- CLOSE
			local CLOSE = WINDOW_MANAGER:CreateControlFromVirtual("LOUTils_CLIPBOARD_ALERT_CLOSE", ALERT, "ZO_CloseButton")
			CLOSE:SetDimensions(16, 16)
			CLOSE:SetAnchor(TOPRIGHT, ALERT, TOPRIGHT, 0, 0)
			CLOSE:SetHandler("OnMouseDown", function(self) 
				_G["LOUTils_CLIPBOARD_ALERT"]:SetHidden(true) 
				_G["LOUTils_CLIPBOARD_ALERT"]:SetTopmost(false) 
			end) 

			-- TITLE
			local TITLE = WINDOW_MANAGER:CreateControlFromVirtual("LOUTils_CLIPBOARD_ALERT_TITLE", ALERT, "ZO_WindowTitle")
			TITLE:SetHorizontalAlignment(CENTER)
			TITLE:ClearAnchors()
			TITLE:SetAnchor(TOPLEFT, ALERT, TOPLEFT, 0, -1)
			TITLE:SetAnchor(TOPRIGHT, ALERT, TOPRIGHT, 0, -1)
			TITLE:SetText("PARTIAL COPY AND PASTE!")
			
			-- DIVIDER
			local DIVIDER = WINDOW_MANAGER:CreateControl("LOUTils_CLIPBOARD_ALERT_DIVIDER", ALERT, CT_TEXTURE)
			DIVIDER:SetTexture("EsoUI/Art/Miscellaneous/horizontalDivider.dds")
			DIVIDER:SetDimensions(nil, 4)
			DIVIDER:SetAnchor(TOPLEFT, ALERT, TOPLEFT, -80, 40)
			DIVIDER:SetAnchor(TOPRIGHT, ALERT, TOPRIGHT, 80, 40)
			
			-- TEXT
			local TEXT = WINDOW_MANAGER:CreateControl("LOUTils_CLIPBOARD_ALERT_TEXT", ALERT, CT_LABEL)
			TEXT:SetFont("ZoFontGame")
			TEXT:SetAnchor(CENTER, ALERT, CENTER)
			TEXT:SetColor(207/255, 220/255, 189/255, 1)
			
			-- NEXT
			local NEXT = WINDOW_MANAGER:CreateControlFromVirtual("LOUTils_CLIPBOARD_ALERT_NEXT", ALERT, "ZO_DefaultButton")
			NEXT:SetAnchor(BOTTOMLEFT, ALERT, BOTTOMLEFT, 0, -10)
			NEXT:SetText("NEXT")
			NEXT:SetHandler("OnMouseDown", function(self) 
				CALLBACK_MANAGER:FireCallbacks("LOUTils_OnClipBoardCopyNext", self)
			end) 
			
			-- CANCEL
			local CANCEL = WINDOW_MANAGER:CreateControlFromVirtual("LOUTils_CLIPBOARD_ALERT_CANCEL", ALERT, "ZO_DefaultButton")
			CANCEL:SetAnchor(BOTTOMRIGHT, ALERT, BOTTOMRIGHT, 0, -10)
			CANCEL:SetText("CANCEL")
			CANCEL:SetHandler("OnMouseDown", function(self) 
				_G["LOUTils_CLIPBOARD_ALERT"]:SetHidden(true) 
				_G["LOUTils_CLIPBOARD_ALERT"]:SetTopmost(false) 
			end)
		end
	
		local dumped_parts = self:str_lensplit(text, 1000)
		
		_G["LOUTils_CLIPBOARD_ALERT"]:SetHidden(false)
		_G["LOUTils_CLIPBOARD_ALERT"]:SetTopmost(true)
		
		local i = 1
		local num_max = #dumped_parts
						
		local function LOUTils_OnClipBoardCopyNext()
			if(i > num_max) then 
				i = 1
				_G["LOUTils_CLIPBOARD_ALERT"]:SetHidden(true)
				_G["LOUTils_CLIPBOARD_ALERT"]:SetTopmost(false)	
			end
		
			if(dumped_parts[i]) then 
				_G["LOUTils_CLIPBOARD_ALERT_TEXT"]:SetText('Text copied to clipboard ('..tostring(i)..' of '..tostring(num_max)..') ...')
			
				_G["LOUTils_CLIPBOARD_CONTENT"]:Clear()
				_G["LOUTils_CLIPBOARD_CONTENT"]:SetText(dumped_parts[i])
				_G["LOUTils_CLIPBOARD_CONTENT"]:CopyAllTextToClipboard()
				_G["LOUTils_CLIPBOARD_CONTENT"]:Clear()								
			end
			
			i = i + 1					
		end
		
		CALLBACK_MANAGER:RegisterCallback("LOUTils_OnClipBoardCopyNext", LOUTils_OnClipBoardCopyNext)
		LOUTils_OnClipBoardCopyNext()
	
	-- less or equal to 1000 chars only a single copy is needed
	else
		_G["LOUTils_CLIPBOARD_CONTENT"]:Clear()
		_G["LOUTils_CLIPBOARD_CONTENT"]:SetText(text)
		_G["LOUTils_CLIPBOARD_CONTENT"]:CopyAllTextToClipboard()
		_G["LOUTils_CLIPBOARD_CONTENT"]:Clear()
	end
	
	d("Text copied to clipboard.")
	
	return true
end

